"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createMeshHTTPHandler = void 0;
const graphql_1 = require("graphql");
const graphql_yoga_1 = require("graphql-yoga");
const fusion_runtime_1 = require("@graphql-mesh/fusion-runtime");
// eslint-disable-next-line import/no-extraneous-dependencies
const utils_1 = require("@graphql-mesh/utils");
const federation_1 = require("@graphql-tools/federation");
const utils_2 = require("@graphql-tools/utils");
const server_1 = require("@whatwg-node/server");
const http_transport_1 = require("./http-transport");
function createMeshHTTPHandler(config) {
    let transportImportFn;
    if (config.transports != null) {
        if (typeof config.transports === 'function') {
            transportImportFn = config.transports;
        }
        else {
            transportImportFn = (handlerName) => config.transports[handlerName];
        }
    }
    else {
        transportImportFn = defaultTransportImport;
    }
    let fetchAPI = config.fetchAPI;
    // eslint-disable-next-line prefer-const
    let logger;
    // TODO: Move these to other packages later
    function defaultTransportImport(handlerName) {
        if (handlerName === 'http') {
            return http_transport_1.httpTransport;
        }
        if (handlerName === 'rest') {
            const omnigraphPackageName = '@omnigraph/json-schema';
            return Promise.resolve(`${omnigraphPackageName}`).then(s => __importStar(require(s)));
        }
        const omnigraphPackageName = `@omnigraph/${handlerName}`;
        return Promise.resolve(`${omnigraphPackageName}`).then(s => __importStar(require(s)));
    }
    const supergraphSpec = config.spec || 'fusion';
    let supergraphYogaPlugin;
    if (supergraphSpec === 'fusion') {
        supergraphYogaPlugin = (0, fusion_runtime_1.useSupergraph)({
            getSupergraph() {
                if (typeof config.supergraph === 'function') {
                    return config.supergraph();
                }
                if ((0, graphql_1.isSchema)(config.supergraph)) {
                    return config.supergraph;
                }
                if (typeof config.supergraph === 'string') {
                    let url = config.supergraph;
                    if (url.startsWith('.')) {
                        url = `file:///${url}`;
                    }
                    yoga.logger.info(`Fetching Supergraph from ${url}`);
                    return fetchAPI
                        .fetch(config.supergraph, {
                        headers: config.schemaHeaders,
                    })
                        .then(res => res.text())
                        .then(schemaString => (0, graphql_1.buildSchema)(schemaString, {
                        assumeValid: true,
                        assumeValidSDL: true,
                        noLocation: true,
                    }));
                }
                return (0, graphql_1.buildASTSchema)(config.supergraph, {
                    assumeValid: true,
                    assumeValidSDL: true,
                });
            },
            getTransportExecutor(transportEntry, getSubgraph) {
                function handleImportResult(importRes) {
                    const getSubgraphExecutor = importRes.getSubgraphExecutor;
                    if (!getSubgraphExecutor) {
                        throw (0, utils_2.createGraphQLError)(`getSubgraphExecutor is not exported from the transport: ${transportEntry.kind}`);
                    }
                    return getSubgraphExecutor(transportEntry, getSubgraph);
                }
                logger.info(`Loading transport: ${transportEntry.kind}`);
                const importRes$ = transportImportFn(transportEntry.kind);
                if ((0, server_1.isPromise)(importRes$)) {
                    return importRes$.then(handleImportResult);
                }
                return handleImportResult(importRes$);
            },
            polling: config.polling,
        });
    }
    else if (supergraphSpec === 'federation') {
        let supergraph;
        // eslint-disable-next-line no-inner-declarations
        function getAndSetSupergraph() {
            function handleSupergraphInput(supergraphSdl) {
                supergraph = (0, federation_1.getStitchedSchemaFromSupergraphSdl)({
                    supergraphSdl: (0, graphql_1.isSchema)(supergraphSdl)
                        ? (0, utils_2.getDocumentNodeFromSchema)(supergraphSdl)
                        : supergraphSdl,
                });
            }
            if (typeof config.supergraph === 'function') {
                const supergraph$ = config.supergraph();
                if ((0, server_1.isPromise)(supergraph$)) {
                    return supergraph$.then(handleSupergraphInput);
                }
                return handleSupergraphInput(supergraph$);
            }
            if ((0, server_1.isPromise)(config.supergraph)) {
                return config.supergraph.then(handleSupergraphInput);
            }
            return handleSupergraphInput(config.supergraph);
        }
        let initialSupergraph$;
        supergraphYogaPlugin = {
            onRequestParse() {
                return {
                    onRequestParseDone() {
                        initialSupergraph$ ||= getAndSetSupergraph();
                        return initialSupergraph$;
                    },
                };
            },
            onEnveloped({ setSchema }) {
                setSchema(supergraph);
            },
            invalidateSupergraph() {
                return getAndSetSupergraph();
            },
        };
    }
    const yoga = (0, graphql_yoga_1.createYoga)({
        fetchAPI: config.fetchAPI,
        logging: config.logging == null ? new utils_1.DefaultLogger() : config.logging,
        plugins: [...(config.plugins || []), supergraphYogaPlugin],
        context({ request, req, connectionParams }) {
            // Maybe Node-like environment
            if (req?.headers) {
                return {
                    fetch: fetchAPI.fetch,
                    logger,
                    headers: (0, utils_1.getHeadersObj)(req.headers),
                    connectionParams,
                };
            }
            // Fetch environment
            if (request?.headers) {
                return {
                    fetch: fetchAPI.fetch,
                    logger,
                    headers: (0, utils_1.getHeadersObj)(request.headers),
                    connectionParams,
                };
            }
            return {};
        },
        cors: config.cors,
        graphiql: config.graphiql,
        batching: config.batching,
    });
    fetchAPI ||= yoga.fetchAPI;
    logger = yoga.logger;
    Object.defineProperty(yoga, 'invalidateSupergraph', {
        value: supergraphYogaPlugin.invalidateSupergraph,
        configurable: true,
    });
    return yoga;
}
exports.createMeshHTTPHandler = createMeshHTTPHandler;
